xquery version "3.1";

module namespace removal="https://mollenhauer-edition.de/ns/conversions/removal";

(:~  
 : Convert Module Removal
 : *******************************************************************************************
 : This module contains functions to strip the TEI from elements that are not needed for text processing.
 :  
 : @version 1.0 (2021-09-23)
 : @status stable
 : @author Uwe Sikora, Max Zeterberg
 :)

declare namespace tei = "http://www.tei-c.org/ns/1.0";
declare namespace output="http://www.w3.org/2010/xslt-xquery-serialization";

declare option output:method "xml";

(:~
 : Default return for removal:removal: If $node is an elment node it will be reconstructed and its child nodes will 
 : be passed on to removal:removal. If it is not an element, its child nodes will be passed on to removal:removal.
 :
 : @author Max-Ferdinand Zeterberg
 : @author Uwe Sikora
 : @version 1.0
 :  
 : @param $node The current node
:)
declare function removal:default-removal($node as node()*)
{
    try {
        element {QName('http://www.tei-c.org/ns/1.0', $node/name())} {
            $node/@*,
            removal:removal($node/node())
        }
    } catch * {
        removal:removal($node/node())
    }
};

(:~
 : This function deletes the tags of certain elements. Other Elements will be reconstructed, Text will be normalized.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :  
 : @param $node The current node
:)
declare function removal:delete-element-tags($nodes as node()*)
{
    for $node in $nodes
    return
        if (local-name($node) = ('date', 'foreign', 'hi', 'name', 'rs')) then
            removal:delete-element-tags($node/node())
        else
            try {
                element {QName('http://www.tei-c.org/ns/1.0', $node/name())} {
                    $node/@*,
                    removal:removal($node/node())
                }
            } catch * {
                normalize-space($node)
            }
};

(:~
 : This function strips the TEI-Document from all nodes that should not be converted to plain text. It also surrounds
 : tei:seg/@type="margin" and tei:note with whitespace and replaces tei:pause with whitespace.
 :
 : @author Max-Ferdinand Zeterberg
 : @author Uwe Sikora
 : @version 1.0
 :  
 : @param $nodes TEI-Document without critical apparatus.
:)
declare function removal:removal($nodes as node()*) as item()*
{
    for $node in $nodes
    return
        typeswitch ($node)
            case text()
                return
                    ($node)
            case processing-instruction()
                return
                    ()
            case comment()
                return
                    ()
            case element(tei:abbr)
                return
                    if ($node/parent::tei:choice) then
                        ()
                    else
                        removal:default-removal($node)
                        case element(tei:anchor)
                return
                    ()
            case element(tei:bibl)
                return
                    if ($node/@type = "commentary") then
                        ()
                    else
                        removal:default-removal($node)
            case element(tei:date)
                return
                    (removal:delete-element-tags($node/node()))
            case element(tei:facsimile)
                return
                    ()
            case element(tei:figDesc)
                return
                    ()
            case element(tei:foreign)
                return
                    (removal:delete-element-tags($node/node()))
            case element(tei:hi)
                return
                    (removal:delete-element-tags($node/node()))
            case element(tei:label)
                return
                    if ($node/@type = 'note_anchor') then
                        ()
                    else
                        removal:default-removal($node)
            case element(tei:name)
                return
                    (removal:delete-element-tags($node/node()))
            case element(tei:note)
                return
                    if ($node/@type = "commentary") then
                        ()
                    else
                        (" ", removal:default-removal($node), " ")
            case element(tei:pause)
                return
                    ' '
            case element(tei:pb)
                return
                    ()
            case element(tei:rs)
                return
                    (removal:delete-element-tags($node/node()))
            case element(tei:seg)
                return
                    if ($node/@type = "eyecatcher") then
                        removal:default-removal($node)
                    else
                        if ($node/@type = "margin") then
                            (" ", removal:default-removal($node), " ")
                        else
                            ()
            case element(tei:teiHeader)
                return
                    ()
            case element(tei:text)
                return
                    if ($node/@type = "commentary") then
                        ()
                    else
                        removal:default-removal($node)
            default
                return
                    removal:default-removal($node)
};

(:~
 : This function strips the TEI-Document from all tei:sic.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :  
 : @param $nodes TEI-Document without critical apparatus.
:)
declare function removal:delete-sic($nodes as node()*) as item()*
{
    
    for $node in $nodes
    return
        typeswitch ($node)
            case text()
                return
                    ($node)
            case element(tei:sic)
                return
                    ()
            default
                return
                    try {
                        element {QName('http://www.tei-c.org/ns/1.0', $node/name())} {
                            $node/@*,
                            removal:delete-sic($node/node())
                        }
                    } catch * {
                        removal:delete-sic($node/node())
                    }
};

(:~
 : This function strips the TEI-Document from all tei:corr.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :  
 : @param $nodes TEI-Document without critical apparatus.
:)
declare function removal:delete-corr($nodes as node()*) as item()*
{
    
    for $node in $nodes
    return
        typeswitch ($node)
            case text()
                return
                    ($node)
            case element(tei:corr)
                return
                    ()
            default
                return
                    try {
                        element {QName('http://www.tei-c.org/ns/1.0', $node/name())} {
                            $node/@*,
                            removal:delete-corr($node/node())
                        }
                    } catch * {
                        removal:delete-corr($node/node())
                    }
};