xquery version "3.1";

module namespace selection="https://mollenhauer-edition.de/ns/conversions/selection";

declare namespace tei = "http://www.tei-c.org/ns/1.0";

(:~
 : This function checks if the TEI-Document has the same title as $title.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $doc: TEI-Document
 : @param $title: Exact title of the Text --> //tei:titleStmt/tei:title[@type='main']
 : @return: True or False
 :)
declare function selection:checkTitle($doc as node(), $title as xs:string*)
{
    codepoint-equal(normalize-space($doc//tei:titleStmt/tei:title[@type = 'main']), normalize-space($title))
};

(:~
 : This function checks if the TEI-Document has been published the first time in $year.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $doc: TEI-Document
 : @param $year: Year in which the Text was published first -->
 : //tei:witness[1]/tei:listWit[1]/tei:witness[1]/tei:desc[1]/text())
 : @return: True or False
 :)
declare function selection:checkYear($doc as node(), $year as xs:string*)
{
    matches(normalize-space($doc//tei:witness[1]/tei:listWit[1]/tei:witness[1]/tei:desc[1]),
            normalize-space($year))
};

(:~
 : This function returns the filename or the filenames:
 :  --> If only one file exist that matches title, year, or title and year, then only one filname will be returned.
 :  --> If more than one file matches title, year, or title and year, then more than one filname will be returned.
 :  --> If none file matches title, year, or title and year, then an empty sequence will be returned.
 : This function calls the functions selection:checkTitle and selection:checkYear.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $path: directory where the file is stored
 : @param $title: Exact title of the Text --> //tei:titleStmt/tei:title[@type='main']
 : @param $year: Year in which the Text was published first -->
 : //tei:witness[1]/tei:listWit[1]/tei:witness[1]/tei:desc[1]/text())
 : @return: A sequence with either one, more or zero items --> filename(s)
 :)

declare function selection:getFileNames($path as xs:string, $titles as xs:string*, $years as xs:string*)
{
    if (exists($titles)) then
        for $title in $titles
        return
            if (exists($years)) then
                for $year in $years
                return
                    for $doc in collection($path)
                    where (selection:checkTitle($doc, $title) and selection:checkYear($doc, $year))
                    return
                        util:document-name($doc)
            else
                for $doc in collection($path)
                where (selection:checkTitle($doc, $title))
                return
                    util:document-name($doc)
    else if (exists($years)) then
        for $year in $years
                return
                    for $doc in collection($path)
                    where (selection:checkYear($doc, $year))
                    return
                        util:document-name($doc)
    else
        for $doc in collection($path)
        return
            util:document-name($doc)
};

(:~
 : This function checks if the Value of $fassung exists as tei:witness.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $doc: TEI-Document
 : @param $fassung: Fassung of the KMG-Text -->
 : //tei:sourceDesc[1]/tei:listWit[1]/tei:witness[@*[namespace-uri() = 'http://www.w3.org/XML/1998/namespace' and
 :             local-name() = 'id']
 : @return: True or False
 :)
declare function selection:check_fassung($doc as node()*, $fassung as xs:string)
{
    exists($doc//tei:sourceDesc[1]/tei:listWit[1]/tei:witness[@*[namespace-uri() =
               'http://www.w3.org/XML/1998/namespace' and local-name() = 'id'] = $fassung])
};


